@@ -55,7 +55,10 @@ def to_internal(option_spec, parsed_options):
     internal = AttributeDict()
     for k, v in option_spec.items():
         if not v.get('metavar', ''):
-            internal[k] = False
+            if v.get('multiple', ''):
+                internal[k] = []
+            else:
+                internal[k] = False
     options_used = AttributeDict()
     values_used = AttributeDict()
     position = AttributeDict()
@@ -73,7 +76,8 @@ def to_internal(option_spec, parsed_options):
                         )
                     )
                 # Make this check optional at some point
-                if options_used.has_key(int_var):
+                multiple = opt.get('multiple', False)
+                if options_used.has_key(int_var) and not multiple:
                     raise getopt.GetoptError(
                         'The option %r is unexpected (the related option %r '
                         'has already been used)'%( 
@@ -83,9 +87,18 @@ def to_internal(option_spec, parsed_options):
                     )
                 else:
                     if opt.get('metavar', ''):
-                        internal[int_var] = value
+                        if multiple:
+                            if not internal.has_key(int_var):
+                                internal[int_var] = [value]
+                            else:
+                                internal[int_var].append(value)
+                        else:
+                            internal[int_var] = value
                     else:
-                        internal[int_var] = True
+                        if multiple:
+                            internal[int_var].append(True)
+                        else:
+                            internal[int_var] = True
                     if options_used.has_key(int_var):
                         options_used[int_var].append(option)
                         values_used[int_var].append(value)
@@ -110,19 +123,19 @@ def check_command(name, command):
                 command.arg_spec,
             )
         )
-    #if name is not None and command.arg_spec:
-    #    raise Exception(
-    #        'Was not expecting key \'args\' for the sub-command %r'%(name,)
-    #    )
-    for k, opt in command.option_spec.items():
-        for options in opt.get('options', []):
-            if not options.startswith('-'):
-                raise Exception(
-                    'Expected the %r option for the %r variable in command '
-                    '%r to start with the characters \'--\' or \'-\''%(
-                        options, k, name
+    if isinstance(command, dict):
+        for name_, command_ in command.items():
+            check_command(name_, command_)
+    else:
+        for k, opt in command.option_spec.items():
+            for options in opt.get('options', []):
+                if not options.startswith('-'):
+                    raise Exception(
+                        'Expected the %r option for the %r variable in command '
+                        '%r to start with the characters \'--\' or \'-\''%(
+                            options, k, name
+                        )
                     )
-                )
 
 #
 # Help helpers
@@ -152,14 +165,17 @@ def two_cols(rows):
 def usage(commands, name, program):
     help = 'Usage: %s '%program
     if commands[None].option_spec:
-        help += '[GLOBAL_OPTIONS] '
+        help += '[OPTIONS] '
     if commands[None].arg_spec:
         for parts in commands[None].arg_spec:
             if isinstance(parts[0], int):
                 help += parts[3]+' '
             else:
                 help += parts[0]+' '
-    help += 'COMMAND '
+    if name is None:
+        help += 'COMMAND '
+    else:
+        help += name+' '
     if name is not None:
         help += '[OPTIONS] '
         if commands[name].arg_spec:
@@ -188,7 +204,7 @@ def global_options(commands, name, program):
                     cur_opts.append('%s %s'%(option, metavar))
         rows.append((' '.join(cur_opts), opt.get('help', '')))
     if rows:
-        help += 'Global options:\n'
+        help += 'Options:\n'
         help += two_cols(rows)
     return help
 
@@ -237,12 +253,20 @@ def sub_commands(commands, name, program):
     rows = []
     for sub_command in commands.keys():
         if sub_command is not None:
-            rows.append(
-                (
-                    sub_command, 
-                    commands[sub_command].help.get('summary', '')
+            if isinstance(commands[sub_command], dict):
+                rows.append(
+                    (
+                        sub_command, 
+                        commands[sub_command][None].help.get('summary', '')
+                    )
+                )
+            else:
+                rows.append(
+                    (
+                        sub_command, 
+                        commands[sub_command].help.get('summary', '')
+                    )
                 )
-            )
     if rows:
         help += 'Commands:\n'
         help += two_cols(rows)
@@ -295,10 +319,6 @@ def assemble_help(
                 template = """%(summary)s
 %(usage)s
 
-%(global_options)s
-
-%(global_args)s
-
 %(sub_command_options)s
 
 %(sub_command_args)s
@@ -334,7 +354,11 @@ def process_command(cmd_line_parts, name, command):
     new_cmd_line_parts = cmd_line_parts[:]
     main_short = ''
     main_long = []
-    for k, opt in command.option_spec.items():
+    if isinstance(command, dict):
+        to_loop = command[None].option_spec.items()
+    else:
+        to_loop = command.option_spec.items()
+    for k, opt in to_loop:
         for options in opt.get('options', []):
             if options.startswith('--'):
                 add = opt.get('metavar') and '=' or ''
@@ -436,7 +460,7 @@ class Cmd(object):
             elif len(args) > 1:
                 raise getopt.GetoptError(
                     'Unexpected arguments: %s'%(
-                        ', '.join([str(arg) for arg in args])[:-1]
+                        ', '.join([str(arg) for arg in args])[:]
                     )
                 )
             elif len(args):
@@ -467,8 +491,6 @@ class Cmd(object):
     def on_config(self, service, args, opts, processed):
         return 0
 
-
-
 class LoggingCmd(Cmd):
 
     help = {
@@ -548,6 +570,7 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None):
             help = assemble_help(commands, None, program)
             commands[None].on_display_help(service, help)
             return 0
+        # Need to be smarter here, want to call the inner functions in the scope of the outer.
         result = commands[None].on_run(service, k, p)
         if result:
             return result
@@ -561,32 +584,41 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None):
             raise getopt.GetoptError(error)
         #if main_opts_used:
         #    new_cmd_line_parts.pop(new_cmd_line_parts.index(sub_command))
+        if isinstance(commands[sub_command], dict):
+            return handle_command(commands[sub_command], cmd_line_parts=args, program='%s %s'%(program, sub_command), service=service)
         sub_args, sub_opts = process_command(
             args,#new_cmd_line_parts, 
             sub_command, 
             commands[sub_command],
         )
-        if not hasattr(commands[sub_command], 'on_initial_convert'):
-            raise Exception('The command for %r has no %r attribute'%(sub_command, 'on_initial_convert'))
-        k, p, h = commands[sub_command].on_initial_convert(
-            service, 
-            commands[sub_command].option_spec, 
-            sub_args, 
-            sub_opts,
-        )
-        if h:
-            help = assemble_help(
-                commands, 
-                sub_command, 
-                program, 
+            #help = assemble_help(
+            #    commands[sub_command], 
+            #    None, 
+            #    '%s %s'%(program, sub_command) 
+            #)
+            #commands[sub_command][None].on_display_help(service, help)
+            #return 0
+        if 1:#else:
+            if not hasattr(commands[sub_command], 'on_initial_convert'):
+                raise Exception('The command for %r has no %r attribute'%(sub_command, 'on_initial_convert'))
+            k, p, h = commands[sub_command].on_initial_convert(
+                service, 
+                commands[sub_command].option_spec, 
+                sub_args, 
+                sub_opts,
             )
-            commands[None].on_display_help(service, help)
-            return 0
-        result = commands[sub_command].on_run(service, k, p)
+            if h:
+                help = assemble_help(
+                    commands, 
+                    sub_command, 
+                    program, 
+                )
+                commands[None].on_display_help(service, help)
+                return 0
+            result = commands[sub_command].on_run(service, k, p)
     except getopt.GetoptError, err:
         print 'Error:', str(err)
         if sub_command:
-
             args = commands[None].arg_spec and ' %s'%(' '.join([arg[0] for arg in commands[None].arg_spec]))
             if args == []:
                 args = ''
@@ -598,10 +630,11 @@ def handle_command(commands, cmd_line_parts=None, program=None, service=None):
                 'sub_command': sub_command,
                 'args': args,
             }
+            return 1
         else:
             print "Try `%(program)s --help' for more information." % {
                 'program': program,
             }
-    else:
-        return result or 0
+            return 1
+    return result or 0
 
